@@ -8,14 +8,32 @@ from dataclasses import dataclass, field
 from enum import Enum
 from typing import Any, Callable, Generic, TypeVar
 
-from sqlalchemy import bindparam, delete, insert, select, update
+from sqlalchemy import bindparam, insert, select, update
 
+from ..cascade import has_cascade_delete_relations
+from ..internal import ResultProcessor
 from ..session import AsyncSession, ctx_session
+from .upsert import ConflictResolution as UpsertConflictResolution
+from .upsert import UpsertHandler
 
 
 T = TypeVar("T")
 
 
+def _get_error_code(exception: Exception) -> str:
+    """Extract error code from database exception."""
+    error_str = str(exception).lower()
+    if "unique" in error_str or "duplicate" in error_str:
+        return "unique_violation"
+    elif "not null" in error_str:
+        return "not_null_violation"
+    elif "foreign key" in error_str:
+        return "foreign_key_violation"
+    elif "check constraint" in error_str:
+        return "check_violation"
+    return "unknown_error"
+
+
 class TransactionMode(Enum):
     """Transaction modes for bulk operations."""
 
@@ -127,69 +145,6 @@ class BulkTransactionManager:
             return await operation_func(self.session, batch_data)
 
 
-class DatabaseNativeHandler:
-    """Handles database-specific conflict resolution."""
-
-    def __init__(self, dialect_name: str):
-        self.dialect_name = dialect_name
-
-    def get_conflict_clause(
-        self, conflict_resolution: ConflictResolution, conflict_fields: list[str] | None = None
-    ) -> str | None:
-        """Get database-specific conflict resolution clause."""
-        if self.dialect_name == "postgresql":
-            if conflict_resolution == ConflictResolution.IGNORE:
-                fields = f"({', '.join(conflict_fields)})" if conflict_fields else ""
-                return f"ON CONFLICT {fields} DO NOTHING"
-            elif conflict_resolution == ConflictResolution.UPDATE and conflict_fields:
-                updates = ", ".join(f"{field} = EXCLUDED.{field}" for field in conflict_fields)
-                return f"ON CONFLICT ({', '.join(conflict_fields)}) DO UPDATE SET {updates}"
-        elif self.dialect_name == "mysql":
-            if conflict_resolution == ConflictResolution.IGNORE:
-                return "INSERT IGNORE"
-            elif conflict_resolution == ConflictResolution.UPDATE:
-                return "INSERT ... ON DUPLICATE KEY UPDATE"
-        elif self.dialect_name == "sqlite":
-            if conflict_resolution == ConflictResolution.IGNORE:
-                return "INSERT OR IGNORE"
-            elif conflict_resolution == ConflictResolution.UPDATE:
-                return "INSERT OR REPLACE"
-        return None
-
-    def modify_statement(self, stmt, conflict_resolution: ConflictResolution, conflict_fields: list[str] | None = None):
-        """Modify SQL statement for conflict resolution."""
-        if conflict_resolution == ConflictResolution.IGNORE:
-            if self.dialect_name == "postgresql" and hasattr(stmt, "on_conflict_do_nothing"):
-                if conflict_fields:
-                    return stmt.on_conflict_do_nothing(index_elements=conflict_fields)
-                return stmt.on_conflict_do_nothing()
-            elif self.dialect_name == "sqlite":
-                # For SQLite, use INSERT OR IGNORE
-                return stmt.prefix_with("OR IGNORE")
-            elif self.dialect_name == "mysql":
-                # For MySQL, use INSERT IGNORE
-                return stmt.prefix_with("IGNORE")
-        elif conflict_resolution == ConflictResolution.UPDATE:
-            if self.dialect_name == "sqlite":
-                # For SQLite, use INSERT OR REPLACE
-                return stmt.prefix_with("OR REPLACE")
-        return stmt
-
-    @staticmethod
-    def get_error_code(exception: Exception) -> str:
-        """Extract error code from database exception."""
-        error_str = str(exception).lower()
-        if "unique" in error_str or "duplicate" in error_str:
-            return "unique_violation"
-        elif "not null" in error_str:
-            return "not_null_violation"
-        elif "foreign key" in error_str:
-            return "foreign_key_violation"
-        elif "check constraint" in error_str:
-            return "check_violation"
-        return "unknown_error"
-
-
 class BulkOperationHandler:
     """Unified handler for bulk operations with SQLAlchemy native capabilities."""
 
@@ -198,7 +153,6 @@ class BulkOperationHandler:
         self.table = table
         self.model_class = model_class
         self.dialect = session.bind.dialect
-        self.native_handler = DatabaseNativeHandler(self.dialect.name)
         self.transaction_manager = BulkTransactionManager(session, transaction_mode)
         self._current_return_fields: list[str] | None = None
 
@@ -238,6 +192,9 @@ class BulkOperationHandler:
                 failed_batches += 1
                 if on_error == ErrorHandling.FAIL_FAST:
                     raise
+                elif on_error == ErrorHandling.IGNORE:
+                    # Silently skip failed batch
+                    pass
                 elif on_error == ErrorHandling.COLLECT:
                     for j, item in enumerate(batch):
                         all_failed_records.append(FailedRecord(i + j, item, e, "batch_error", batch_count - 1))
@@ -291,23 +248,12 @@ class BulkOperationHandler:
     def get_return_columns(self, return_fields: list[str] | None):
         """Get columns to return based on return_fields parameter."""
         if return_fields:
-            return [self.table.c[field] for field in return_fields]
+            return [self.table.c[f] for f in return_fields]
         return list(self.table.c)
 
     def create_objects_from_rows(self, rows, return_fields: list[str] | None = None) -> list:
-        """Create model objects or dictionaries from database rows."""
-        objects = []
-        for row in rows:
-            row_dict = dict(row._mapping)  # noqa
-            if return_fields:
-                # When return_fields is specified, return dict with only requested fields
-                filtered_dict = {field: row_dict.get(field) for field in return_fields}
-                objects.append(filtered_dict)
-            else:
-                # When return_fields is None, return full model objects
-                obj = self.model_class.from_dict(row_dict, validate=False)
-                objects.append(obj)
-        return objects
+        """Create model objects or dictionaries from database rows using ResultProcessor."""
+        return ResultProcessor.rows_to_objects(rows, self.model_class, return_fields)
 
     async def execute_with_returning(
         self,
@@ -323,19 +269,16 @@ class BulkOperationHandler:
         exec_session = session or self.session
 
         if return_columns and self.supports_returning(operation):
-            try:
-                stmt_with_returning = stmt.returning(*return_columns)
-                # For INSERT operations, use the data directly as parameters
-                if operation == "insert" and isinstance(parameters, list):
-                    result = await exec_session.execute(stmt_with_returning, parameters)
-                elif parameters:
-                    result = await exec_session.execute(stmt_with_returning, parameters)
-                else:
-                    result = await exec_session.execute(stmt_with_returning)
-                objects = self.create_objects_from_rows(result.fetchall(), return_fields)
-                return objects, result.rowcount or 0, True
-            except Exception:  # noqa
-                pass  # Fall through to regular execution
+            stmt_with_returning = stmt.returning(*return_columns)
+            # For INSERT operations, use the data directly as parameters
+            if operation == "insert" and isinstance(parameters, list):
+                result = await exec_session.execute(stmt_with_returning, parameters)
+            elif parameters:
+                result = await exec_session.execute(stmt_with_returning, parameters)
+            else:
+                result = await exec_session.execute(stmt_with_returning)
+            objects = self.create_objects_from_rows(result.fetchall(), return_fields)
+            return objects, result.rowcount or 0, True
 
         # Regular execution without RETURNING
         if parameters is not None and isinstance(parameters, list) and len(parameters) > 1:
@@ -344,7 +287,14 @@ class BulkOperationHandler:
         else:
             # Use regular execute for single parameter set or no parameters
             result = await exec_session.execute(stmt, parameters)
-        return [], result.rowcount or 0, False
+
+        # Handle rowcount: -1 means unknown, use parameter count as fallback
+        rowcount = result.rowcount if result.rowcount >= 0 else 0  # noqa
+        if rowcount == 0 and isinstance(parameters, list):
+            # For INSERT/UPDATE without proper rowcount, use parameter count
+            rowcount = len(parameters)
+
+        return [], rowcount, False
 
     async def execute_with_error_handling(
         self,
@@ -361,19 +311,47 @@ class BulkOperationHandler:
         # Use provided session or fall back to instance session
         exec_session = session or self.session
 
-        # Modify statement for conflict resolution
+        # Use unified UPSERT handler for conflict resolution
         if conflict_resolution != ConflictResolution.ERROR:
-            stmt = self.native_handler.modify_statement(stmt, conflict_resolution, conflict_fields)
+            upsert_handler = UpsertHandler(exec_session)
+            upsert_conflict = (
+                UpsertConflictResolution.IGNORE
+                if conflict_resolution == ConflictResolution.IGNORE
+                else UpsertConflictResolution.UPDATE
+            )
+            # For INSERT operations, replace the statement with UPSERT
+            if operation == "insert":
+                stmt = upsert_handler.get_upsert_statement(self.table, data_batch, upsert_conflict, conflict_fields)
 
         successful_objects = []
         failed_records = []
         success_count = 0
 
-        if error_handling in (ErrorHandling.FAIL_FAST, ErrorHandling.IGNORE):
-            # Unified handling for FAIL_FAST and IGNORE modes
+        if error_handling == ErrorHandling.FAIL_FAST:
+            # FAIL_FAST: Execute batch and let exceptions propagate
             successful_objects, success_count = await self._execute_batch_operation(
                 stmt, operation, data_batch, return_columns, exec_session
             )
+        elif error_handling == ErrorHandling.IGNORE:
+            # IGNORE: Process individually to skip errors
+            for _, data in enumerate(data_batch):
+                try:
+                    single_stmt = stmt.values([data]) if hasattr(stmt, "values") else stmt
+                    if return_columns and self.supports_returning(operation):
+                        single_stmt = single_stmt.returning(*return_columns)
+                        result = await exec_session.execute(single_stmt)
+                        rows = result.fetchall()
+                        if rows:
+                            current_return_fields = getattr(self, "_current_return_fields", None)
+                            successful_objects.extend(self.create_objects_from_rows(rows, current_return_fields))
+                            success_count += 1
+                    else:
+                        result = await exec_session.execute(single_stmt)
+                        if result.rowcount > 0:  # noqa
+                            success_count += 1
+                except Exception:  # noqa
+                    # Silently ignore errors
+                    pass
 
         elif error_handling == ErrorHandling.COLLECT:
             # Process individually to collect detailed errors
@@ -393,7 +371,7 @@ class BulkOperationHandler:
                         if result.rowcount > 0:  # noqa
                             success_count += 1
                 except Exception as e:
-                    failed_records.append(FailedRecord(i, data, e, self.native_handler.get_error_code(e)))
+                    failed_records.append(FailedRecord(i, data, e, _get_error_code(e)))
 
         return successful_objects, success_count, failed_records
 
@@ -481,7 +459,15 @@ class BulkOperationHandler:
         """Handle UPDATE operations without RETURNING support."""
         # Execute update first
         result = await session.execute(stmt, data_batch)
-        success_count = result.rowcount or 0
+        success_count = result.rowcount
+
+        # If rowcount is -1 (unknown), execute individually to get accurate count
+        if success_count < 0:  # noqa
+            success_count = 0
+            for data in data_batch:
+                individual_result = await session.execute(stmt, [data])
+                if individual_result.rowcount > 0:  # noqa
+                    success_count += individual_result.rowcount
 
         # If successful, create objects with merged data
         if success_count > 0:
@@ -507,7 +493,7 @@ class BulkOperationHandler:
                 current_return_fields = getattr(self, "_current_return_fields", None)
                 if current_return_fields:
                     # Return only requested fields as dict
-                    filtered_dict = {field: obj_data.get(field) for field in current_return_fields}
+                    filtered_dict = {f: obj_data.get(f) for f in current_return_fields}
                     successful_objects.append(filtered_dict)
                 else:
                     # Return full model object
@@ -587,15 +573,63 @@ async def bulk_create(
     session = manager._get_session(readonly=False)  # noqa
     handler = BulkOperationHandler(session, manager._table, manager._model_class, transaction_mode)  # noqa
 
-    stmt = insert(manager._table)  # noqa
+    # Process data to exclude non-insertable fields
+    processed_objects = []
+    for obj_data in objects:
+        processed_item = {}
+        for key, value in obj_data.items():
+            if hasattr(manager._model_class, key):  # noqa
+                # Create a temporary instance to use the _should_exclude_from_insert method
+                temp_instance = manager._model_class()  # noqa
+                if not temp_instance._should_exclude_from_insert(key, value):  # noqa
+                    processed_item[key] = value
+            else:
+                # If field doesn't exist on model, include it (let database handle the error)
+                processed_item[key] = value
+        processed_objects.append(processed_item)
+
     return_columns = handler.get_return_columns(return_fields) if return_objects else None
 
     async def operation_func(session_, batch_data):
-        # Store return_fields in handler for use in create_objects_from_rows
-        handler._current_return_fields = return_fields
-        return await handler.execute_with_error_handling(
-            stmt, "insert", batch_data, on_error, on_conflict, conflict_fields, return_columns, session_
-        )
+        # Use unified UPSERT handler for conflict resolution
+        if on_conflict != ConflictResolution.ERROR:
+            upsert_handler = UpsertHandler(session_)
+            upsert_conflict = (
+                UpsertConflictResolution.IGNORE
+                if on_conflict == ConflictResolution.IGNORE
+                else UpsertConflictResolution.UPDATE
+            )
+
+            # Store return_fields in handler for use in create_objects_from_rows
+            handler._current_return_fields = return_fields
+
+            # Execute UPSERT with returning
+            batch_results = await upsert_handler.execute_upsert_with_returning(
+                manager._table,  # noqa
+                batch_data,
+                upsert_conflict,
+                conflict_fields,  # noqa
+            )
+
+            # Convert results to model instances only if return_objects is True
+            operator_successful_objects = []
+            if return_objects:
+                for result_data in batch_results:
+                    if return_fields:
+                        filtered_dict = {f: result_data.get(f) for f in return_fields}
+                        operator_successful_objects.append(filtered_dict)
+                    else:
+                        instance = manager._model_class.from_dict(result_data)  # noqa
+                        operator_successful_objects.append(instance)
+
+            return operator_successful_objects, len(batch_results), []
+        else:
+            # Regular insert without conflict resolution
+            stmt = insert(manager._table)  # noqa
+            handler._current_return_fields = return_fields
+            return await handler.execute_with_error_handling(
+                stmt, "insert", batch_data, on_error, on_conflict, conflict_fields, return_columns, session_
+            )
 
     (
         successful_objects,
@@ -604,13 +638,13 @@ async def bulk_create(
         batch_count,
         failed_batches,
         rollback_count,
-    ) = await handler.execute_bulk_operation(objects, batch_size, operation_func, transaction_mode, on_error)
+    ) = await handler.execute_bulk_operation(processed_objects, batch_size, operation_func, transaction_mode, on_error)
 
     return handler.build_result(
         successful_objects,
         total_success_count,
         failed_records,
-        len(objects),
+        len(processed_objects),
         transaction_mode,
         batch_count,
         failed_batches,
@@ -631,6 +665,7 @@ async def bulk_update(
     on_error: ErrorHandling = ErrorHandling.FAIL_FAST,
     on_conflict: ConflictResolution = ConflictResolution.ERROR,
     conflict_fields: list[str] | None = None,
+    accurate_count: bool = True,
 ) -> int | list | BulkResult:
     """Perform true bulk update operations for better performance.
 
@@ -645,6 +680,9 @@ async def bulk_update(
         on_error: Error handling strategy
         on_conflict: Conflict resolution strategy
         conflict_fields: Fields to check for conflicts
+        accurate_count: Whether to return accurate update count (default: True)
+            - True: Guarantees accurate count (PostgreSQL uses individual execution, slower)
+            - False: Prioritizes performance, may return approximate count (PostgreSQL ~20x faster)
 
     Returns:
         - int: Number of updated records (default, backward compatible)
@@ -667,39 +705,69 @@ async def bulk_update(
     session = manager._get_session(readonly=False)  # noqa
     handler = BulkOperationHandler(session, manager._table, manager._model_class, transaction_mode)  # noqa
 
-    # Build base statement with conflict resolution
+    # Fast path: use executemany for all databases when accurate_count=False
+    if not accurate_count:
+        return await _bulk_update_fast(manager, mappings, match_fields, batch_size, session)
+
+    # Build base statement
     where_conditions = [manager._table.c[field] == bindparam(f"match::{field}") for field in match_fields]  # noqa
     stmt = update(manager._table).where(*where_conditions)  # noqa
 
-    # Apply conflict resolution if specified
-    if on_conflict != ConflictResolution.ERROR:
-        stmt = handler.native_handler.modify_statement(stmt, on_conflict, conflict_fields)
-
     async def operation_func(session_, batch_data):
-        # Add update values
-        update_values = {key: bindparam(f"update::{key}") for key in batch_data[0].keys() if key not in match_fields}
-        if not update_values:
-            return [], 0, []
+        # Use unified UPSERT handler for bulk update with conflict resolution
+        if on_conflict != ConflictResolution.ERROR:
+            upsert_handler = UpsertHandler(session_)
+            upsert_conflict = UpsertConflictResolution.UPDATE
 
-        batch_stmt = stmt.values(**update_values)
+            # Store return_fields in handler for use in create_objects_from_rows
+            handler._current_return_fields = return_fields
 
-        # Prepare parameter mappings
-        param_mappings = []
-        for mapping in batch_data:
-            param_dict = {}
-            for f in match_fields:
-                param_dict[f"match::{f}"] = mapping[f]
-            for key, value in mapping.items():
-                if key not in match_fields:
-                    param_dict[f"update::{key}"] = value
-            param_mappings.append(param_dict)
+            # Execute UPSERT with returning
+            batch_results = await upsert_handler.execute_upsert_with_returning(
+                manager._table,  # noqa
+                batch_data,
+                upsert_conflict,
+                match_fields,  # noqa
+            )
 
-        return_columns = handler.get_return_columns(return_fields) if return_objects else None
-        # Store return_fields in handler for use in create_objects_from_rows
-        handler._current_return_fields = return_fields
-        return await handler.execute_with_error_handling(
-            batch_stmt, "update", param_mappings, on_error, on_conflict, conflict_fields, return_columns, session_
-        )
+            # Convert results to model instances only if return_objects is True
+            oper_successful_objects = []
+            if return_objects:
+                for result_data in batch_results:
+                    if return_fields:
+                        filtered_dict = {field: result_data.get(field) for field in return_fields}
+                        oper_successful_objects.append(filtered_dict)
+                    else:
+                        instance = manager._model_class.from_dict(result_data)  # noqa
+                        oper_successful_objects.append(instance)
+
+            return oper_successful_objects, len(batch_results), []
+        else:
+            # Regular update without conflict resolution
+            update_values = {
+                key: bindparam(f"update::{key}") for key in batch_data[0].keys() if key not in match_fields
+            }
+            if not update_values:
+                return [], 0, []
+
+            batch_stmt = stmt.values(**update_values)
+
+            # Prepare parameter mappings
+            param_mappings = []
+            for mapping in batch_data:
+                param_dict = {}
+                for f in match_fields:
+                    param_dict[f"match::{f}"] = mapping[f]
+                for key, value in mapping.items():
+                    if key not in match_fields:
+                        param_dict[f"update::{key}"] = value
+                param_mappings.append(param_dict)
+
+            return_columns = handler.get_return_columns(return_fields) if return_objects else None
+            handler._current_return_fields = return_fields
+            return await handler.execute_with_error_handling(
+                batch_stmt, "update", param_mappings, on_error, on_conflict, conflict_fields, return_columns, session_
+            )
 
     (
         successful_objects,
@@ -765,30 +833,58 @@ async def bulk_delete(
     handler = BulkOperationHandler(session, manager._table, manager._model_class, transaction_mode)  # noqa
 
     async def operation_func(session_, batch_ids):
+        from sqlalchemy import delete
+
+        # Check if model has cascade delete relationships
+        has_cascade = has_cascade_delete_relations(manager._model_class)
+
+        # Fetch instances if needed for return_objects or cascade processing
+        instances = []
+        if return_objects or has_cascade:
+            field_column = manager._table.c[id_field]  # noqa
+            in_condition = field_column.in_(batch_ids)
+
+            # Handle FunctionExpression by resolving to SQLAlchemy expression
+            if hasattr(in_condition, "resolve"):
+                in_condition = in_condition.resolve(manager._table)  # noqa
+
+            select_stmt = select(manager._table).where(in_condition)  # noqa
+            result = await session_.execute(select_stmt)
+
+            for row in result.fetchall():
+                instance = manager._model_class.from_dict(dict(row._mapping))  # noqa
+                instances.append(instance)
+
+        # Process cascade deletion if needed
+        if has_cascade:
+            from ..cascade import CascadeExecutor
+
+            handler._current_return_fields = return_fields
+            cascade_executor = CascadeExecutor()
+            for instance in instances:
+                await cascade_executor._delete_related_objects(instance, session_)  # noqa
+
+        # Execute batch delete with single SQL statement
         field_column = manager._table.c[id_field]  # noqa
         in_condition = field_column.in_(batch_ids)
 
-        # Handle FunctionExpression by resolving to SQLAlchemy expression
         if hasattr(in_condition, "resolve"):
             in_condition = in_condition.resolve(manager._table)  # noqa
 
         stmt = delete(manager._table).where(in_condition)  # noqa
-        return_columns = handler.get_return_columns(return_fields) if return_objects else None
+        result = await session_.execute(stmt)
 
-        if return_objects and not handler.supports_returning("delete"):
-            # Fallback: select before delete
-            # Store return_fields in handler for use in create_objects_from_rows
-            handler._current_return_fields = return_fields
-            objects_batch = await handler.handle_select_fallback(in_condition, return_columns, session_)
-            result = await session_.execute(stmt)
-            return objects_batch, result.rowcount or 0, []
-        else:
-            # Store return_fields in handler for use in create_objects_from_rows
-            handler._current_return_fields = return_fields
-            objects, rowcount, _ = await handler.execute_with_returning(
-                stmt, "delete", return_columns, None, session_, return_fields
-            )
-            return objects, rowcount, []
+        # Build return objects if requested
+        objects_batch = []
+        if return_objects:
+            for instance in instances:
+                if return_fields:
+                    filtered_dict = {f: getattr(instance, f, None) for f in return_fields}
+                    objects_batch.append(filtered_dict)
+                else:
+                    objects_batch.append(instance)
+
+        return objects_batch, result.rowcount, []
 
     (
         successful_objects,
@@ -810,3 +906,46 @@ async def bulk_delete(
         rollback_count,
         return_objects,
     )
+
+
+async def _bulk_update_fast(
+    manager, mappings: list[dict[str, Any]], match_fields: list[str], batch_size: int, session: AsyncSession
+) -> int:
+    """Fast bulk update using executemany (may return approximate count for PostgreSQL)."""
+    if not mappings:
+        return 0
+
+    # Build UPDATE statement
+    where_conditions = [manager._table.c[f] == bindparam(f"match::{f}") for f in match_fields]  # noqa
+    stmt = update(manager._table).where(*where_conditions)  # noqa
+
+    # Get update fields (exclude match fields)
+    update_values = {key: bindparam(f"update::{key}") for key in mappings[0].keys() if key not in match_fields}
+    if not update_values:
+        return 0
+
+    stmt = stmt.values(**update_values)
+
+    # Process in batches
+    total = 0
+    for i in range(0, len(mappings), batch_size):
+        batch = mappings[i : i + batch_size]
+
+        # Prepare parameter mappings
+        param_mappings = []
+        for mapping in batch:
+            param_dict = {}
+            for f in match_fields:
+                param_dict[f"match::{f}"] = mapping[f]
+            for key, value in mapping.items():
+                if key not in match_fields:
+                    param_dict[f"update::{key}"] = value
+            param_mappings.append(param_dict)
+
+        # Execute with executemany
+        result = await session.execute(stmt, param_mappings)
+        # PostgreSQL returns -1, use batch length as approximation
+        count = result.rowcount if result.rowcount >= 0 else len(batch)  # noqa
+        total += count
+
+    return total
